#!/usr/bin/env python
# -*- coding: utf-8 -*-

from utils import *
from nltk.corpus import wordnet

class WSDClassifier(object):
    '''
    Abstract class for WSD classifier
    '''

    def __init__(self, lemma, sense_inventory):
        self.lemma = lemma
        self.sense_inventory = sense_inventory

    def train(self, examples):
        raise NotImplementedError

    def predict_sense(self, example): 
        raise NotImplementedError

    def evaluate_on_test_set(self, examples, dump_predicted_senses=False):
        acc = 0
        correct = 0
        total = len(examples)
        for example in examples:
            s = self.predict_sense(example)
            if dump_predicted_senses:
                print 'pred=', s, 'gold=', example.sense()
            if s == example.sense():
                correct += 1
        acc = round(correct/float(total),2)
        return acc


class MostFrequentSenseBaseline(WSDClassifier):
    '''
    Most Frequent Sense baseline
    '''

    def __init__(self, lemma, sense_inventory):
        WSDClassifier.__init__(self, lemma, sense_inventory)
        self.mfs = None

    def train(self, examples):
        # ....
        pass

    def predict_sense(self, example):
        # ....
        pass



class WNFirstSenseBaseline(WSDClassifier):
    '''
    Wordnet first sense baseline
    '''
    def __init__(self, lemma, sense_inventory):
        WSDClassifier.__init__(self, lemma, sense_inventory)
        # ...

    def train(self, examples):
        # ....
        pass

    def predict_sense(self, example):
        # ....
        pass
    

    
        
class SimplifiedLesk(WSDClassifier):
    '''
    Simplified Lesk WSD algorithm
    '''

    def __init__(self, lemma, sense_inventory, mfs=None):
        WSDClassifier.__init__(self, lemma, sense_inventory)
        self.mfs = mfs
        self.signatures = {}

    def train(self, examples):
        # ....
        pass

    def predict_sense(self, example):
        # ....
        pass
    



class NaiveBayes(WSDClassifier):
    '''
    Naive Bayes classifier 
    '''
    def __init__(self, lemma, sense_inventory):
        WSDClassifier.__init__(self, lemma, sense_inventory)
        # ...

    def train(self, examples):
        # ....
        pass

    def predict_sense(self, example):
        # ....
        pass


###############################################################



###############################################################


if __name__ == '__main__':
    
    from twa import WSDExamples
    from optparse import OptionParser

    usage = """ Test de differentes methodes de WSD ...
                utilisant des donnees au format TWS sense tagged data
                %prog TWA_FILE
"""
    parser=OptionParser(usage=usage)
    (opts,args) = parser.parse_args()
    if (len(args) > 0):
        sensed_tagged_data_file = args[0]
    else:
        exit(usage+'\nIl manque en argument un fichier TWA...\n')

    # chargement des exemples annotés avec sens
    examples = WSDExamples(sensed_tagged_data_file).examples
    print "Distribution des sens:", sense_distribution(examples)

    # lemma
    lemma = examples[0].lemma()

    # l'inventaire des sens
    possible_senses = list(set([example.sense() for example in examples]))

    # split des exemples en donnees d'apprentissage / de test
    train, test = data_split(examples)

    # entraîner et appliquer les classifieurs
    # ...
    
    
    
    






